import h5py
import scipy.sparse as sparse
import numpy as np

def chr_pair_h5loader(h5_path: str, chr_pair: str, resolution = int(4096e3), batch = int(1e6)) -> sparse.csr_matrix:
    h5_list = [h5_path] if type(h5_path) is str else h5_path
    f_it = True
    rev = False
    for h5_p in h5_list:
        with h5py.File(h5_p, "a") as h5file:
            if chr_pair not in h5file['data'].keys():
                chr_pair_new = f"{chr_pair.split('-')[1]}-{chr_pair.split('-')[0]}"
                rev = True
                if chr_pair_new not in h5file['data'].keys():
                    raise KeyError(f'Chromosome pairs {chr_pair} and {chr_pair_new} are not in {h5_path}')
                else:
                    chr_pair = chr_pair_new
            assert h5file['data'][chr_pair]['row'].shape[0] == h5file['data'][chr_pair]['col'].shape[0], f'Broken data! Len of row and col coords should be the same'
            chr1_size = int(np.ceil(h5file['chr_sizes'][chr_pair.split('-')[0]][0]/resolution))
            chr2_size = int(np.ceil(h5file['chr_sizes'][chr_pair.split('-')[1]][0]/resolution))
            if f_it:
                mat = sparse.csr_matrix((chr1_size, chr2_size), dtype=np.int)
                f_it = False

            if batch < h5file['data'][chr_pair]['row'].shape[0]:
                for i in range(int(np.ceil(h5file['data'][chr_pair]['row'].shape[0]/batch))):
                    row = np.array(h5file['data'][chr_pair]['row'][i*batch : (i+1)*batch], dtype=np.int)//resolution
                    col = np.array(h5file['data'][chr_pair]['col'][i*batch : (i+1)*batch], dtype=np.int)//resolution
                    mat = mat + sparse.csr_matrix((np.ones_like(row, dtype=np.int), (row, col)), shape=(chr1_size, chr2_size))
            else:
                row = np.array(h5file['data'][chr_pair]['row'][:], dtype=np.int)//resolution
                col = np.array(h5file['data'][chr_pair]['col'][:], dtype=np.int)//resolution
                mat += sparse.csr_matrix((np.ones_like(row, dtype=np.int), (row, col)), shape=(chr1_size, chr2_size))

    if rev: mat = mat.T 
    return mat